@@ -1,6 +1,7 @@
 import time
 import string
 import logging
+from functools import wraps
 from collections import namedtuple, OrderedDict
 from contextlib import contextmanager
 
@@ -72,13 +73,26 @@ class Record(object):
         self._xodb_weight = weight
         self._xodb_query = query
         self._xodb_db = db
+        self._loaded = False
 
     @lazy_property
     def _xodb_schema(self):
         typ, data = loads(self._xodb_document.get_data())
+        self._loaded = True
         return _lookup_schema(typ).from_flat(data)
 
     def __getattr__(self, name):
+        if self._xodb_db.use_values and not self._loaded:
+            # short circuit expensive schema loading,
+            # if value is available
+            sort = self._xodb_db.value_sorts.get(name)
+            # TODO: date, datetime
+            if sort and sort in ('integer', 'string'):
+                num = self._xodb_db.values[name]
+                val = self._xodb_document.get_value(num)
+                if sort == 'integer':
+                    val = xapian.sortable_unserialise(val)
+                return val
         try:
             return self._xodb_schema[name].value
         except KeyError:
@@ -186,6 +200,17 @@ class MultipleValueRangeProcessor(xapian.ValueRangeProcessor):
         return (xapian.BAD_VALUENO, begin, end)
 
 
+def reconnector(func):
+    @wraps(func)
+    def _connect(db, *args, **kwargs):
+        try:
+            return func(db, *args, **kwargs)
+        except xapian.NetworkError:
+            db.reconnect()
+            raise
+    return _connect
+
+                     
 class Database(object):
     """An xodb database.
 
@@ -216,6 +241,7 @@ class Database(object):
     backend = None
     _metadata_keyset = None
     query_cache_limit = 1024
+    use_values = True
 
     @contextmanager
     def transaction(self):
@@ -262,6 +288,7 @@ class Database(object):
         self.query_cache = LRUDict(limit=self.query_cache_limit)
         self.inmem = inmem
         self._value_count = 0
+        self._timeout = 10000
 
         if isinstance(path, basestring):
             if writable:
@@ -280,7 +307,11 @@ class Database(object):
                 raise TypeError("replication can only be used "
                                 "if a database path is provided.")
             self.backend = self.db_path
-
+        elif isinstance(path, tuple):
+            if replicated:
+                raise TypeError("replication can only be used "
+                                "if a database path is provided.")
+            self.reconnect()
         self.reopen()
 
     def get_query_parser(self, language, default_op, check_cache=True, 
@@ -297,9 +328,20 @@ class Database(object):
             self.parsers_by_language[language] = qp
         return qp
 
+    def reconnect(self):
+        if self.backend is not None:
+            self.backend.close()
+            del self.backend
+        if self._writable:
+            self.backend = xapian.remote_open_writable(*self.db_path)
+        else:
+            self.backend = xapian.remote_open(*self.db_path)
+
+    @reconnector
     def close(self):
         self.backend.close()
 
+    @reconnector
     def flush(self):
         self.backend.flush()
 
@@ -753,6 +795,7 @@ class Database(object):
                 if self._writable:
                     self.value_count = 0
 
+    @reconnector
     def querify(self, query,
                 language=None,
                 translit=None,
@@ -799,6 +842,7 @@ class Database(object):
                                         default_op,
                                         parser_flags) for q in query))
 
+    @reconnector
     def query(self, query,
               offset=0,
               limit=0,
@@ -909,6 +953,7 @@ class Database(object):
                 self.reopen()
                 tries += 1
 
+    @reconnector
     def count(self,
               query="",
               language=None,
@@ -932,6 +977,7 @@ class Database(object):
         mset = self._build_mset(enq, retry_limit=retry_limit)
         return mset.size()
 
+    @reconnector
     def facet(self, query,
               prefix='facet',
               estimate=True,
@@ -979,6 +1025,7 @@ class Database(object):
             results[facet] = counter(q, language=language)
         return results
 
+    @reconnector
     def expand(self, query, expand,
                language=None,
                echo=False,
@@ -1024,6 +1071,7 @@ class Database(object):
                 results[(name, score)] = score
         return OrderedDict(sorted(results.items(), key=lambda i: i[0][1], reverse=True))
 
+    @reconnector
     def estimate(self, query,
                  limit=0,
                  klimit=1.0,
@@ -1054,6 +1102,7 @@ class Database(object):
 
         return mset.get_matches_estimated()
 
+    @reconnector
     def term_freq(self, term):
         """
         Return a count of the number of documents indexed for a given
@@ -1062,6 +1111,7 @@ class Database(object):
         self.reopen()
         return self.backend.get_termfreq(term)
 
+    @reconnector
     def describe_query(self, query,
                        language=None,
                        default_op=Query.OP_AND,
@@ -1075,6 +1125,7 @@ class Database(object):
             return qp.parse_query(query, default_parser_flags) 
         return str(self.retry_if_modified(op, retry_limit))
 
+    @reconnector
     def spell(self, query,
               language=None,
               default_op=Query.OP_AND,
@@ -1090,6 +1141,7 @@ class Database(object):
             return qp.get_corrected_query_string().decode('utf8')
         return self.retry_if_modified(op, retry_limit)
 
+    @reconnector
     def suggest(self, query,
                 offset=0,
                 limit=0,
